@@ -1,8 +1,12 @@
+#[cfg(shuttle)]
+use shuttle::sync::atomic::{AtomicUsize, Ordering};
+#[cfg(not(all(shuttle)))]
 use std::sync::atomic::{AtomicUsize, Ordering};
 
 use crossbeam_epoch::Guard;
 
 use crate::{
+    child_ptr::NodePtr,
     lock::{ConcreteReadGuard, ReadGuard, WriteGuard},
     node_16::Node16,
     node_256::Node256,
@@ -15,7 +19,7 @@ pub(crate) const MAX_STORED_PREFIX_LEN: usize = 10;
 pub(crate) type Prefix = [u8; MAX_STORED_PREFIX_LEN];
 
 #[repr(u8)]
-#[derive(Clone, Copy)]
+#[derive(Clone, Copy, PartialEq, Eq, Debug)]
 pub(crate) enum NodeType {
     N4 = 0,
     N16 = 1,
@@ -23,32 +27,9 @@ pub(crate) enum NodeType {
     N256 = 3,
 }
 
-pub(crate) trait Node: Send + Sync {
-    fn new(prefix: &[u8]) -> Box<Self>;
-    fn base(&self) -> &BaseNode;
-    fn base_mut(&mut self) -> &mut BaseNode;
-    fn is_full(&self) -> bool;
-    fn is_under_full(&self) -> bool;
-    fn insert(&mut self, key: u8, node: *const BaseNode);
-    fn change(&mut self, key: u8, val: *const BaseNode);
-    fn get_child(&self, key: u8) -> Option<*const BaseNode>;
-    fn get_children(&self, start: u8, end: u8) -> Result<Vec<(u8, *const BaseNode)>, ()>;
-    fn remove(&mut self, k: u8);
-    fn copy_to<N: Node>(&self, dst: &mut N);
-}
-
-#[repr(C)]
-pub(crate) struct BaseNode {
-    // 2b type | 60b version | 1b lock | 1b obsolete
-    pub(crate) type_version_lock_obsolete: AtomicUsize,
-    pub(crate) prefix_cnt: u32,
-    pub(crate) count: u16, // TODO: we only need u8
-    pub(crate) prefix: Prefix,
-}
-
-impl Drop for BaseNode {
-    fn drop(&mut self) {
-        let layout = match self.get_type() {
+impl NodeType {
+    fn node_layout(&self) -> std::alloc::Layout {
+        match *self {
             NodeType::N4 => std::alloc::Layout::from_size_align(
                 std::mem::size_of::<Node4>(),
                 std::mem::align_of::<Node4>(),
@@ -69,13 +50,102 @@ impl Drop for BaseNode {
                 std::mem::align_of::<Node256>(),
             )
             .unwrap(),
-        };
+        }
+    }
+}
+
+pub(crate) trait Node {
+    fn new(prefix: &[u8]) -> Box<Self>;
+    fn base(&self) -> &BaseNode;
+    fn base_mut(&mut self) -> &mut BaseNode;
+    fn is_full(&self) -> bool;
+    fn is_under_full(&self) -> bool;
+    fn insert(&mut self, key: u8, node: NodePtr);
+    fn change(&mut self, key: u8, val: NodePtr);
+    fn get_child(&self, key: u8) -> Option<NodePtr>;
+    fn get_children(&self, start: u8, end: u8) -> Vec<(u8, NodePtr)>;
+    fn remove(&mut self, k: u8);
+    fn copy_to<N: Node>(&self, dst: &mut N);
+    fn get_type() -> NodeType;
+}
+
+#[repr(C)]
+pub(crate) struct BaseNode {
+    // 2b type | 60b version | 1b lock | 1b obsolete
+    pub(crate) type_version_lock_obsolete: AtomicUsize,
+    pub(crate) prefix_cnt: u32,
+    pub(crate) count: u16, // TODO: we only need u8
+    pub(crate) prefix: Prefix,
+}
+
+impl Drop for BaseNode {
+    fn drop(&mut self) {
+        let layout = self.get_type().node_layout();
         unsafe {
             std::alloc::dealloc(self as *mut BaseNode as *mut u8, layout);
         }
     }
 }
 
+macro_rules! gen_method {
+    ($method_name:ident, ($($arg_n:ident : $args:ty),*), $return:ty) => {
+        impl BaseNode {
+            pub(crate) fn $method_name(&self, $($arg_n : $args),*) -> $return {
+                match self.get_type() {
+                    NodeType::N4 => {
+                        let node = unsafe{&* (self as *const BaseNode as *const Node4)};
+                        node.$method_name($($arg_n),*)
+                    },
+                    NodeType::N16 => {
+                        let node = unsafe{&* (self as *const BaseNode as *const Node16)};
+                        node.$method_name($($arg_n),*)
+                    },
+                    NodeType::N48 => {
+                        let node = unsafe{&* (self as *const BaseNode as *const Node48)};
+                        node.$method_name($($arg_n),*)
+                    },
+                    NodeType::N256 => {
+                        let node = unsafe{&* (self as *const BaseNode as *const Node256)};
+                        node.$method_name($($arg_n),*)
+                    },
+                }
+            }
+        }
+    };
+}
+
+macro_rules! gen_method_mut {
+    ($method_name:ident, ($($arg_n:ident : $args:ty),*), $return:ty) => {
+        impl BaseNode {
+            pub(crate) fn $method_name(&mut self, $($arg_n : $args),*) -> $return {
+                match self.get_type() {
+                    NodeType::N4 => {
+                        let node = unsafe{&mut * (self as *mut BaseNode as *mut Node4)};
+                        node.$method_name($($arg_n),*)
+                    },
+                    NodeType::N16 => {
+                        let node = unsafe{&mut * (self as *mut BaseNode as *mut Node16)};
+                        node.$method_name($($arg_n),*)
+                    },
+                    NodeType::N48 => {
+                        let node = unsafe{&mut * (self as *mut BaseNode as *mut Node48)};
+                        node.$method_name($($arg_n),*)
+                    },
+                    NodeType::N256 => {
+                        let node = unsafe{&mut * (self as *mut BaseNode as *mut Node256)};
+                        node.$method_name($($arg_n),*)
+                    },
+                }
+            }
+        }
+    };
+}
+
+gen_method!(get_child, (k: u8), Option<NodePtr>);
+gen_method!(get_children, (start: u8, end: u8), Vec<(u8, NodePtr)>);
+gen_method_mut!(change, (key: u8, val: NodePtr), ());
+gen_method_mut!(remove, (key: u8), ());
+
 impl BaseNode {
     pub(crate) fn new(n_type: NodeType, prefix: &[u8]) -> Self {
         let val = convert_type_to_version(n_type);
@@ -108,31 +178,16 @@ impl BaseNode {
         unsafe { std::mem::transmute(val as u8) }
     }
 
-    pub(crate) fn set_prefix(&mut self, prefix: *const u8, len: usize) {
-        if len > 0 {
-            let len = std::cmp::min(len, MAX_STORED_PREFIX_LEN);
-            unsafe {
-                std::ptr::copy_nonoverlapping(
-                    prefix,
-                    &mut self.prefix as *mut [u8; MAX_STORED_PREFIX_LEN] as *mut u8,
-                    len,
-                );
-            }
-            self.prefix_cnt = len as u32;
-        } else {
-            self.prefix_cnt = 0;
+    pub(crate) fn set_prefix(&mut self, prefix: &[u8]) {
+        let len = prefix.len();
+        self.prefix_cnt = len as u32;
+
+        for (i, v) in prefix.iter().enumerate() {
+            self.prefix[i] = *v;
         }
     }
 
-    pub(crate) fn read_lock(&self) -> Result<usize, usize> {
-        let version = self.type_version_lock_obsolete.load(Ordering::Acquire);
-        if Self::is_locked(version) || Self::is_obsolete(version) {
-            return Err(version);
-        }
-        Ok(version)
-    }
-
-    pub(crate) fn read_lock_n(&self) -> Result<ReadGuard, usize> {
+    pub(crate) fn read_lock(&self) -> Result<ReadGuard, usize> {
         let version = self.type_version_lock_obsolete.load(Ordering::Acquire);
         if Self::is_locked(version) || Self::is_obsolete(version) {
             return Err(version);
@@ -142,19 +197,9 @@ impl BaseNode {
     }
 
     #[allow(dead_code)]
-    pub(crate) fn write_lock_n(&self) -> Result<WriteGuard, usize> {
-        let read = self.read_lock_n()?;
-        read.upgrade_to_write_lock().map_err(|v| v.1)
-    }
-
-    /// returns need restart
-    pub(crate) fn read_unlock(&self, start_read: usize) -> Result<usize, ()> {
-        let version = self.type_version_lock_obsolete.load(Ordering::Acquire);
-        if start_read != version {
-            Err(())
-        } else {
-            Ok(version)
-        }
+    pub(crate) fn write_lock(&self) -> Result<WriteGuard, usize> {
+        let read = self.read_lock()?;
+        read.upgrade().map_err(|v| v.1)
     }
 
     fn is_locked(version: usize) -> bool {
@@ -173,79 +218,12 @@ impl BaseNode {
         self.prefix_cnt > 0
     }
 
-    pub(crate) fn get_prefix_len(&self) -> u32 {
+    pub(crate) fn prefix_len(&self) -> u32 {
         self.prefix_cnt
     }
 
-    pub(crate) fn get_prefix(&self) -> &[u8] {
-        unsafe { std::slice::from_raw_parts(self.prefix.as_ptr(), self.prefix_cnt as usize) }
-    }
-
-    pub(crate) fn get_child(key: u8, node: &BaseNode) -> Option<*const BaseNode> {
-        match node.get_type() {
-            NodeType::N4 => {
-                let cur_n = unsafe { &*(node as *const BaseNode as *const Node4) };
-                cur_n.get_child(key)
-            }
-            NodeType::N16 => {
-                let cur_n = unsafe { &*(node as *const BaseNode as *const Node16) };
-                cur_n.get_child(key)
-            }
-            NodeType::N48 => {
-                let cur_n = unsafe { &*(node as *const BaseNode as *const Node48) };
-                cur_n.get_child(key)
-            }
-            NodeType::N256 => {
-                let cur_n = unsafe { &*(node as *const BaseNode as *const Node256) };
-                cur_n.get_child(key)
-            }
-        }
-    }
-
-    pub(crate) fn change(node: &mut BaseNode, key: u8, val: *mut BaseNode) {
-        match node.get_type() {
-            NodeType::N4 => {
-                let n = node as *mut BaseNode as *mut Node4;
-                unsafe { &mut *n }.change(key, val);
-            }
-            NodeType::N16 => {
-                let n = node as *mut BaseNode as *mut Node16;
-                unsafe { &mut *n }.change(key, val);
-            }
-            NodeType::N48 => {
-                let n = node as *mut BaseNode as *mut Node48;
-                unsafe { &mut *n }.change(key, val);
-            }
-            NodeType::N256 => {
-                let n = node as *mut BaseNode as *mut Node256;
-                unsafe { &mut *n }.change(key, val);
-            }
-        }
-    }
-
-    pub(crate) fn get_children(
-        node: &BaseNode,
-        start: u8,
-        end: u8,
-    ) -> Result<Vec<(u8, *const BaseNode)>, ()> {
-        match node.get_type() {
-            NodeType::N4 => {
-                let n = node as *const BaseNode as *const Node4;
-                unsafe { &*n }.get_children(start, end)
-            }
-            NodeType::N16 => {
-                let n = node as *const BaseNode as *const Node16;
-                unsafe { &*n }.get_children(start, end)
-            }
-            NodeType::N48 => {
-                let n = node as *const BaseNode as *const Node48;
-                unsafe { &*n }.get_children(start, end)
-            }
-            NodeType::N256 => {
-                let n = node as *const BaseNode as *const Node256;
-                unsafe { &*n }.get_children(start, end)
-            }
-        }
+    pub(crate) fn prefix(&self) -> &[u8] {
+        self.prefix[..self.prefix_cnt as usize].as_ref()
     }
 
     pub(crate) fn insert_grow<CurT: Node, BiggerT: Node>(
@@ -253,7 +231,7 @@ impl BaseNode {
         parent_node: Option<ReadGuard>,
         key_parent: u8,
         key: u8,
-        val: *mut BaseNode,
+        val: NodePtr,
         guard: &Guard,
     ) -> Result<(), ()> {
         if !n.as_ref().is_full() {
@@ -269,30 +247,24 @@ impl BaseNode {
 
         let p = parent_node.expect("parent node must present when current node is full");
 
-        let mut write_p = p.upgrade_to_write_lock().map_err(|_| ())?;
+        let mut write_p = p.upgrade().map_err(|_| ())?;
 
-        let mut write_n = match n.upgrade_to_write_lock() {
-            Ok(w) => w,
-            Err(_) => {
-                return Err(());
-            }
-        };
+        let mut write_n = n.upgrade_to_write_lock().map_err(|_| ())?;
 
-        let mut n_big = BiggerT::new(write_n.as_ref().base().get_prefix());
+        let mut n_big = BiggerT::new(write_n.as_ref().base().prefix());
         write_n.as_ref().copy_to(n_big.as_mut());
         n_big.insert(key, val);
 
-        BaseNode::change(
-            write_p.as_mut(),
+        write_p.as_mut().change(
             key_parent,
-            Box::into_raw(n_big) as *mut BaseNode,
+            NodePtr::from_node(Box::into_raw(n_big) as *mut BaseNode),
         );
 
         write_n.mark_obsolete();
-        let delete_n = unsafe { &mut *(write_n.as_mut() as *mut CurT as *mut BaseNode) };
+        let delete_n = write_n.as_mut() as *mut CurT as usize;
         std::mem::forget(write_n);
         guard.defer(move || unsafe {
-            std::ptr::drop_in_place(delete_n);
+            std::ptr::drop_in_place(delete_n as *mut BaseNode);
         });
         Ok(())
     }
@@ -302,7 +274,7 @@ impl BaseNode {
         parent: Option<ReadGuard>,
         key_parent: u8,
         key: u8,
-        val: *mut BaseNode,
+        val: NodePtr,
         guard: &Guard,
     ) -> Result<(), ()> {
         match node.as_ref().get_type() {
@@ -340,38 +312,4 @@ impl BaseNode {
             ),
         }
     }
-
-    pub(crate) fn is_leaf(ptr: *const BaseNode) -> bool {
-        debug_assert!(!ptr.is_null());
-        (ptr as usize & (1 << 63)) == (1 << 63)
-    }
-
-    pub(crate) fn get_leaf(ptr: *const BaseNode) -> usize {
-        ptr as usize & ((1 << 63) - 1)
-    }
-
-    pub(crate) fn set_leaf(tid: usize) -> *mut BaseNode {
-        (tid | (1 << 63)) as *mut BaseNode
-    }
-
-    pub(crate) fn remove_key(node: *mut BaseNode, key: u8) {
-        match unsafe { &*node }.get_type() {
-            NodeType::N4 => {
-                let n = unsafe { &mut *(node as *mut Node4) };
-                n.remove(key);
-            }
-            NodeType::N16 => {
-                let n = unsafe { &mut *(node as *mut Node16) };
-                n.remove(key);
-            }
-            NodeType::N48 => {
-                let n = unsafe { &mut *(node as *mut Node48) };
-                n.remove(key);
-            }
-            NodeType::N256 => {
-                let n = unsafe { &mut *(node as *mut Node256) };
-                n.remove(key);
-            }
-        }
-    }
 }
