@@ -1,4 +1,4 @@
-use crate::{base_node::BaseNode, key::Key, lock::ReadGuard};
+use crate::{base_node::BaseNode, child_ptr::NodePtr, key::RawKey, lock::ReadGuard};
 
 enum PrefixCheckEqualsResult {
     BothMatch,
@@ -12,7 +12,7 @@ enum PrefixCompareResult {
     Bigger,
 }
 
-pub(crate) struct RangeScan<'a, T: Key> {
+pub(crate) struct RangeScan<'a, T: RawKey> {
     start: &'a T,
     end: &'a T,
     result: &'a mut [usize],
@@ -51,21 +51,21 @@ impl KeyTracker {
         std::intrinsics::bswap(val)
     }
 
-    pub(crate) fn append_prefix(node: *const BaseNode, key_tracker: &KeyTracker) -> KeyTracker {
+    pub(crate) fn append_prefix(node: NodePtr, key_tracker: &KeyTracker) -> KeyTracker {
         let mut cur_key = key_tracker.clone();
-        if BaseNode::is_leaf(node) {
+        if node.is_leaf() {
             cur_key
         } else {
-            let node_ref = unsafe { &*node };
-            for i in 0..node_ref.get_prefix_len() {
-                cur_key.push(node_ref.get_prefix()[i as usize]);
+            let node_ref = unsafe { &*node.as_ptr() };
+            for i in 0..node_ref.prefix_len() {
+                cur_key.push(node_ref.prefix()[i as usize]);
             }
             cur_key
         }
     }
 }
 
-impl<'a, T: Key> RangeScan<'a, T> {
+impl<'a, T: RawKey> RangeScan<'a, T> {
     pub(crate) fn new(
         start: &'a T,
         end: &'a T,
@@ -82,7 +82,7 @@ impl<'a, T: Key> RangeScan<'a, T> {
         }
     }
 
-    fn is_valid_key_pair(&self) -> bool {
+    pub(crate) fn is_valid_key_pair(&self) -> bool {
         self.start < self.end
     }
 
@@ -101,144 +101,115 @@ impl<'a, T: Key> RangeScan<'a, T> {
         false
     }
 
-    pub(crate) fn scan(&mut self) -> Option<usize> {
-        if !self.is_valid_key_pair() {
-            return None;
-        }
+    pub(crate) fn scan(&mut self) -> Result<Option<usize>, usize> {
+        let mut level = 0;
+        let mut node: ReadGuard;
+        let mut next_node = self.root;
+        let mut parent_node: Option<ReadGuard> = None;
+        self.to_continue = 0;
+        self.result_found = 0;
 
-        'outer: loop {
-            let mut level = 0;
-            let mut node: ReadGuard;
-            let mut next_node = self.root;
-            let mut parent_node: Option<ReadGuard> = None;
-            self.to_continue = 0;
-            self.result_found = 0;
+        let mut key_tracker = KeyTracker::default();
 
-            let mut key_tracker = KeyTracker::default();
+        'inner: loop {
+            node = unsafe { &*next_node }.read_lock()?;
 
-            'inner: loop {
-                node = if let Ok(v) = unsafe { &*next_node }.read_lock_n() {
-                    v
-                } else {
-                    continue 'outer;
-                };
+            let prefix_check_result =
+                self.check_prefix_equals(node.as_ref(), &mut level, &mut key_tracker);
 
-                let prefix_check_result =
-                    match self.check_prefix_equals(node.as_ref(), &mut level, &mut key_tracker) {
-                        Ok(v) => v,
-                        Err(_) => continue 'outer,
+            if parent_node.is_some() {
+                parent_node.as_ref().unwrap().check_version()?;
+            }
+
+            node.check_version()?;
+
+            match prefix_check_result {
+                PrefixCheckEqualsResult::BothMatch => {
+                    let start_level = if self.start.len() > level as usize {
+                        self.start.as_bytes()[level as usize]
+                    } else {
+                        0
+                    };
+                    let end_level = if self.end.len() > level as usize {
+                        self.end.as_bytes()[level as usize]
+                    } else {
+                        255
                     };
 
-                if parent_node.is_some() && parent_node.as_ref().unwrap().check_version().is_err() {
-                    continue 'outer;
-                }
+                    if start_level != end_level {
+                        let children = node.as_ref().get_children(start_level, end_level);
+                        node.check_version()?;
 
-                if node.check_version().is_err() {
-                    continue 'outer;
-                }
-
-                match prefix_check_result {
-                    PrefixCheckEqualsResult::BothMatch => {
-                        let start_level = if self.start.len() > level as usize {
-                            self.start.as_bytes()[level as usize]
-                        } else {
-                            0
-                        };
-                        let end_level = if self.end.len() > level as usize {
-                            self.end.as_bytes()[level as usize]
-                        } else {
-                            255
-                        };
-
-                        if start_level != end_level {
-                            let children = if let Ok(val) =
-                                BaseNode::get_children(node.as_ref(), start_level, end_level)
-                            {
-                                val
-                            } else {
-                                continue 'outer;
-                            };
-
-                            for (k, n) in children.iter() {
-                                key_tracker.push(*k);
-                                if *k == start_level {
-                                    if self
-                                        .find_start(*n, level + 1, &node, key_tracker.clone())
-                                        .is_err()
-                                    {
-                                        continue 'outer;
-                                    };
-                                } else if *k > start_level && *k < end_level {
-                                    let cur_key = KeyTracker::append_prefix(*n, &key_tracker);
-                                    if self.copy_node(*n, &cur_key).is_err() {
-                                        continue 'outer;
-                                    };
-                                } else if *k == end_level {
-                                    if self
-                                        .find_end(*n, level + 1, &node, key_tracker.clone())
-                                        .is_err()
-                                    {
-                                        continue 'outer;
-                                    }
-                                }
-                                key_tracker.pop();
-
-                                if self.to_continue > 0 {
-                                    break 'inner;
-                                }
+                        for (k, n) in children.iter() {
+                            key_tracker.push(*k);
+                            if *k == start_level {
+                                self.find_start(*n, level + 1, &node, key_tracker.clone())
+                                    .map_err(|_| 0_usize)?;
+                            } else if *k > start_level && *k < end_level {
+                                let cur_key = KeyTracker::append_prefix(*n, &key_tracker);
+                                self.copy_node(*n, &cur_key).map_err(|_| 0_usize)?;
+                            } else if *k == end_level {
+                                self.find_end(*n, level + 1, &node, key_tracker.clone())
+                                    .map_err(|_| 0_usize)?;
                             }
-                        } else {
-                            next_node = BaseNode::get_child(start_level, node.as_ref())?;
-                            if node.check_version().is_err() {
-                                continue 'outer;
-                            };
+                            key_tracker.pop();
 
-                            key_tracker.push(start_level);
-                            if BaseNode::is_leaf(next_node) {
-                                if self.copy_node(next_node, &key_tracker).is_err() {
-                                    continue 'outer;
-                                };
-                                break;
+                            if self.to_continue > 0 {
+                                break 'inner;
                             }
+                        }
+                    } else {
+                        let next_node_tmp = if let Some(n) = node.as_ref().get_child(start_level) {
+                            n
+                        } else {
+                            return Ok(None);
+                        };
+                        node.check_version()?;
 
-                            level += 1;
-                            parent_node = Some(node);
-                            continue;
+                        key_tracker.push(start_level);
+                        if next_node_tmp.is_leaf() {
+                            self.copy_node(next_node_tmp, &key_tracker)
+                                .map_err(|_| 0_usize)?;
+                            break;
                         }
-                        break;
-                    }
-                    PrefixCheckEqualsResult::Contained => {
-                        if self.copy_node(node.as_ref(), &key_tracker).is_err() {
-                            continue 'outer;
-                        }
-                    }
-                    PrefixCheckEqualsResult::NotMatch => {
-                        return None;
+                        next_node = next_node_tmp.as_ptr();
+
+                        level += 1;
+                        parent_node = Some(node);
+                        continue;
                     }
+                    break;
+                }
+                PrefixCheckEqualsResult::Contained => {
+                    self.copy_node(NodePtr::from_node(node.as_ref()), &key_tracker)
+                        .map_err(|_| 0_usize)?;
+                }
+                PrefixCheckEqualsResult::NotMatch => {
+                    return Ok(None);
                 }
-                break;
             }
+            break;
+        }
 
-            if self.result_found > 0 {
-                return Some(self.result_found);
-            } else {
-                return None;
-            }
+        if self.result_found > 0 {
+            Ok(Some(self.result_found))
+        } else {
+            Ok(None)
         }
     }
 
     fn find_end(
         &mut self,
-        node: *const BaseNode,
+        node: NodePtr,
         mut level: u32,
         parent_node: &ReadGuard,
         mut key_tracker: KeyTracker,
     ) -> Result<(), ()> {
-        if BaseNode::is_leaf(node) {
+        if node.is_leaf() {
             return self.copy_node(node, &key_tracker);
         }
 
-        let node = unsafe { &*node }.read_lock_n().map_err(|_| {})?;
+        let node = unsafe { &*node.as_ptr() }.read_lock().map_err(|_| {})?;
         let prefix_result =
             self.check_prefix_compare(node.as_ref(), self.end, 255, &mut level, &mut key_tracker)?;
 
@@ -254,7 +225,8 @@ impl<'a, T: Key> RangeScan<'a, T> {
                     255
                 };
 
-                let children = BaseNode::get_children(node.as_ref(), 0, end_level)?;
+                let children = node.as_ref().get_children(0, end_level);
+                node.check_version().map_err(|_| ())?;
                 for (k, n) in children.iter() {
                     key_tracker.push(*k);
                     if *k == end_level {
@@ -270,22 +242,24 @@ impl<'a, T: Key> RangeScan<'a, T> {
                 }
                 Ok(())
             }
-            PrefixCompareResult::Smaller => self.copy_node(node.as_ref(), &key_tracker),
+            PrefixCompareResult::Smaller => {
+                self.copy_node(NodePtr::from_node(node.as_ref()), &key_tracker)
+            }
         }
     }
 
     fn find_start(
         &mut self,
-        node: *const BaseNode,
+        node: NodePtr,
         mut level: u32,
         parent_node: &ReadGuard,
         mut key_tracker: KeyTracker,
     ) -> Result<(), ()> {
-        if BaseNode::is_leaf(node) {
+        if node.is_leaf() {
             return self.copy_node(node, &key_tracker);
         }
 
-        let node = unsafe { &*node }.read_lock_n().map_err(|_| {})?;
+        let node = unsafe { &*node.as_ptr() }.read_lock().map_err(|_| {})?;
         let prefix_result =
             self.check_prefix_compare(node.as_ref(), self.start, 0, &mut level, &mut key_tracker)?;
 
@@ -293,14 +267,17 @@ impl<'a, T: Key> RangeScan<'a, T> {
         node.check_version().map_err(|_| {})?;
 
         match prefix_result {
-            PrefixCompareResult::Bigger => self.copy_node(node.as_ref(), &key_tracker),
+            PrefixCompareResult::Bigger => {
+                self.copy_node(NodePtr::from_node(node.as_ref()), &key_tracker)
+            }
             PrefixCompareResult::Equal => {
                 let start_level = if self.start.len() > level as usize {
                     self.start.as_bytes()[level as usize]
                 } else {
                     0
                 };
-                let children = BaseNode::get_children(node.as_ref(), start_level, 255)?;
+                let children = node.as_ref().get_children(start_level, 255);
+                node.check_version().map_err(|_| ())?;
 
                 for (k, n) in children.iter() {
                     key_tracker.push(*k);
@@ -322,21 +299,22 @@ impl<'a, T: Key> RangeScan<'a, T> {
     }
 
     // FIXME: copy node should check parent version to make sure the node is not changed
-    fn copy_node(&mut self, node: *const BaseNode, key_tracker: &KeyTracker) -> Result<(), ()> {
-        if BaseNode::is_leaf(node) {
+    fn copy_node(&mut self, node: NodePtr, key_tracker: &KeyTracker) -> Result<(), ()> {
+        if node.is_leaf() {
             if self.key_in_range(key_tracker) {
                 if self.result_found == self.result.len() {
-                    self.to_continue = BaseNode::get_leaf(node);
+                    self.to_continue = node.as_tid();
                     return Ok(());
                 }
-                self.result[self.result_found] = BaseNode::get_leaf(node);
+                self.result[self.result_found] = node.as_tid();
                 self.result_found += 1;
             };
         } else {
-            let node_ref = unsafe { &*node };
+            let node = unsafe { &*node.as_ptr() }.read_lock().map_err(|_| ())?;
             let mut key_tracker = key_tracker.clone();
 
-            let children = BaseNode::get_children(node_ref, 0, 255)?;
+            let children = node.as_ref().get_children(0, 255);
+            node.check_version().map_err(|_| ())?;
 
             for (k, c) in children.iter() {
                 key_tracker.push(*k);
@@ -363,24 +341,24 @@ impl<'a, T: Key> RangeScan<'a, T> {
         key_tracker: &mut KeyTracker,
     ) -> Result<PrefixCompareResult, ()> {
         if n.has_prefix() {
-            for i in 0..n.get_prefix_len() as usize {
+            for i in 0..n.prefix_len() as usize {
                 let k_level = if k.len() as u32 > *level {
                     k.as_bytes()[*level as usize]
                 } else {
                     fill_key
                 };
 
-                let cur_key = n.get_prefix()[i];
+                let cur_key = n.prefix()[i];
                 key_tracker.push(cur_key);
 
                 if cur_key < k_level {
-                    for j in (i + 1)..n.get_prefix_len() as usize {
-                        key_tracker.push(n.get_prefix()[j]);
+                    for j in (i + 1)..n.prefix_len() as usize {
+                        key_tracker.push(n.prefix()[j]);
                     }
                     return Ok(PrefixCompareResult::Smaller);
                 } else if cur_key > k_level {
-                    for j in (i + 1)..n.get_prefix_len() as usize {
-                        key_tracker.push(n.get_prefix()[j]);
+                    for j in (i + 1)..n.prefix_len() as usize {
+                        key_tracker.push(n.prefix()[j]);
                     }
                     return Ok(PrefixCompareResult::Bigger);
                 }
@@ -396,9 +374,9 @@ impl<'a, T: Key> RangeScan<'a, T> {
         n: &BaseNode,
         level: &mut u32,
         key_tracker: &mut KeyTracker,
-    ) -> Result<PrefixCheckEqualsResult, ()> {
+    ) -> PrefixCheckEqualsResult {
         if n.has_prefix() {
-            for i in 0..n.get_prefix_len() as usize {
+            for i in 0..n.prefix_len() as usize {
                 let start_level = if self.start.len() as u32 > *level {
                     self.start.as_bytes()[*level as usize]
                 } else {
@@ -411,7 +389,7 @@ impl<'a, T: Key> RangeScan<'a, T> {
                     255
                 };
 
-                let cur_key = n.get_prefix()[i as usize];
+                let cur_key = n.prefix()[i as usize];
 
                 if (cur_key == start_level) && (cur_key == end_level) {
                     *level += 1;
@@ -419,15 +397,15 @@ impl<'a, T: Key> RangeScan<'a, T> {
                     continue;
                 } else if (cur_key >= start_level) && (cur_key <= end_level) {
                     key_tracker.push(cur_key);
-                    for j in (i + 1)..n.get_prefix_len() as usize {
-                        key_tracker.push(n.get_prefix()[j]);
+                    for j in (i + 1)..n.prefix_len() as usize {
+                        key_tracker.push(n.prefix()[j]);
                     }
-                    return Ok(PrefixCheckEqualsResult::Contained);
+                    return PrefixCheckEqualsResult::Contained;
                 } else if cur_key < start_level || cur_key > end_level {
-                    return Ok(PrefixCheckEqualsResult::NotMatch);
+                    return PrefixCheckEqualsResult::NotMatch;
                 }
             }
         }
-        Ok(PrefixCheckEqualsResult::BothMatch)
+        PrefixCheckEqualsResult::BothMatch
     }
 }
